def postInstall(context):
    """Called as at the end of the setup process. """
    
    site = context.getSite()
    if hasattr(site, "portal_tinymce"):
        site.portal_tinymce.customplugins = '\n'.join((line for line in site.portal_tinymce.customplugins.splitlines() if line != "plominofield|/plomino_plugins/plominofield/editor_plugin.js"))



from Products.CMFPlomino.PlominoAction import ACTION_TYPES, ACTION_DISPLAY

class PlominoAction(object):
    """
    """
    
    def __init__(self, context, request):
        """Initialize adapter."""
        self.context = context
        self.request = request
        
    def __call__(self):
        """
        """
        return self
        
    def setActionProperties(self):
        """Set field properties to their new values. 
        """
        
        title = self.request.get("actiontitle", self.context.id)
        actionType = self.request.get("actiontype", 'OPENFORM')
        actionDisplay = self.request.get("actiondisplay", 'LINK')
        content = self.request.get("actioncontent", '')
        hideWhen = self.request.get("actionhidewhen", '')
        inActionBar = self.request.get("actioninactionbar", None) == 'on'
        
        # self.context is the current field
        self.context.setTitle(title)
        self.context.setActionType(actionType)
        self.context.setActionDisplay(actionDisplay)
        self.context.setContent(content)
        self.context.setHidewhen(hideWhen)
        self.context.setInActionBar(inActionBar)
        
        self.context.aq_inner.at_post_edit_script()
    
        self.request.RESPONSE.redirect(self.context.absolute_url() + "/../@@tinymceplominoform/valid_page?type=action&value=" + self.context.id)
        


class PlominoCache(object):
    """
    """
    
    def __init__(self, context, request):
        """Initialize adapter."""
        self.context = context
        self.request = request
        
    def __call__(self):
        """
        """
        return self

    def setCacheProperties(self):
        """Set PlominoCache properties to their new values. 
        """
        cacheformula = self.request.get("cacheformula", '')
        
        # self.context is the current hide-when
        self.context.setFormula(cacheformula)
        self.context.aq_inner.at_post_edit_script()
        
        self.request.RESPONSE.redirect(self.context.absolute_url() + "/../@@tinymceplominoform/valid_page?type=cache&value=" + self.context.id)



# -*- coding: utf-8 -*-
# Generated by the windmill services transformer
from windmill.authoring import WindmillTestClient

"""These tests must be loaded by Windmill.
Command example: $ windmill shell firefox http://localhost:8080 loadtest=field_creation.py

CAUTION: there is a bug in Windmill 1.3 while changing the value of a select.
The new value of a select that must be changed is not set. It causes a bug in
the test suite, which can't finish normally.

SOLUTION: After the script is loaded, and before launching it, change manually
the value of this select (the only one), setting its value (val) to CLOSE.
"""

def test_login():
    """Some parameters must be modified (user, password, Plone URL).
    """
    client = WindmillTestClient(__name__)

    client.click(link=u'View your Plone site.')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(timeout=u'8000', id=u'__ac_name')
    client.type(text=u'admin', id=u'__ac_name') # To replace with the corrects username and password
    client.type(text=u'password', id=u'__ac_password')
    client.click(name=u'submit')
    client.waits.forPageLoad(timeout=u'20000')

def test_createDB():
    client = WindmillTestClient(__name__)

    client.click(link=u'Home')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(xpath=u"//dl[@id='plone-contentmenu-factories']/dt/a/span[1]", timeout=u'8000')
    client.click(xpath=u"//dl[@id='plone-contentmenu-factories']/dt/a/span[1]")
    client.click(xpath=u"//a[@id='plominodatabase']/span")
    client.waits.forPageLoad(timeout=u'20000')
    client.type(text=u'testdb', id=u'title')
    client.click(name=u'form.button.save')
    client.waits.forPageLoad(timeout=u'20000')

def test_createForm():
    client = WindmillTestClient(__name__)

    client.click(xpath=u"//dl[@id='plone-contentmenu-factories']/dt/a/span[1]")
    client.click(xpath=u"//a[@id='plominoform']/span")
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(timeout=u'8000', id=u'id')
    client.type(text=u'testform', id=u'id')
    client.type(text=u'testform', id=u'title')
    client.click(name=u'form.button.save')
    client.waits.forPageLoad(timeout=u'20000')

def test_createField():
    client = WindmillTestClient(__name__)

    client.click(link=u'Edit')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(xpath=u"//body[@id='content']", timeout=u'8000')
    client.click(xpath=u"//body[@id='content']")
    client.click(id=u'FormLayout_plominofield')
    client.waits.forElement(timeout=u'5000', id=u'plominoFieldId')
    client.type(text=u'field1', id=u'plominoFieldId')
    client.click(id=u'insert')
    client.click(name=u'form.button.save')
    client.waits.forPageLoad(timeout=u'20000')

def test_createAction():
    client = WindmillTestClient(__name__)

    client.waits.forElement(link=u'Edit', timeout=u'8000')
    client.click(link=u'Edit')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(xpath=u"//body[@id='content']", timeout=u'8000')
    client.click(xpath=u"//body[@id='content']/p")
    client.click(xpath=u"//a[@id='FormLayout_plominoaction']/img")
    client.waits.forElement(timeout=u'5000', id=u'actionid')
    client.type(text=u'action1', id=u'actionid')
    client.select(val=u'CLOSE', id=u'actionTypesList')
    client.click(id=u'insert')
    client.click(name=u'form.button.save')
    client.waits.forPageLoad(timeout=u'20000')

def test_createHidewhen():
    client = WindmillTestClient(__name__)

    client.click(link=u'Edit')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(xpath=u"//body[@id='content']", timeout=u'8000')
    client.click(xpath=u"//body[@id='content']")
    client.click(id=u'FormLayout_plominohidewhen')
    client.waits.forElement(timeout=u'5000', id=u'hidewhenid')
    client.type(text=u'hidewhen1', id=u'hidewhenid')
    client.waits.forElement(timeout=u'5000', id=u'hidewhenFormula')
    client.type(text=u'False', id=u'hidewhenFormula')
    client.click(id=u'insert')
    client.click(name=u'form.button.save')
    client.waits.forPageLoad(timeout=u'20000')

def test_fieldCreated():
    client = WindmillTestClient(__name__)

    client.click(link=u'Contents')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(xpath=u"//a[contains(@href, 'testform/field1')]", timeout=u'8000')

def test_actionCreated():
    client = WindmillTestClient(__name__)

    client.click(link=u'Contents')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(xpath=u"//a[contains(@href, 'testform/action1')]", timeout=u'8000')

def test_hidewhenCreated():
    client = WindmillTestClient(__name__)

    client.click(link=u'Contents')
    client.waits.forPageLoad(timeout=u'20000')
    client.waits.forElement(xpath=u"//a[contains(@href, 'testform/hidewhen1')]", timeout=u'8000')



class PlominoField(object):
    """
    """
    
    def __init__(self, context, request):
        """Initialize adapter."""
        self.context = context
        self.request = request
        
    def __call__(self):
        """
        """
        return self

    def setFieldProperties(self):
        """Set field properties to their new values. 
        """
        
        fieldtype = self.request.get("fieldtype", 'TEXT')
        fieldmode = self.request.get("fieldmode", 'EDITABLE')
        fieldformula = self.request.get("fieldformula", '')
        
        # self.context is the current field
        self.context.setFieldType(fieldtype)
        self.context.setFieldMode(fieldmode)
        self.context.setFormula(fieldformula)
        self.context.aq_inner.at_post_edit_script()
        
        self.request.RESPONSE.redirect(self.context.absolute_url() + "/../@@tinymceplominoform/valid_page?type=field&value=" + self.context.id)



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Products.CMFPlomino.config import FIELD_MODES
from Products.CMFPlomino.PlominoField import get_field_types
from Products.CMFPlomino.PlominoAction import ACTION_TYPES, ACTION_DISPLAY
from Products.CMFPlomino.PlominoAction import PlominoAction
from Products.CMFPlomino.PlominoField import PlominoField
from Products.CMFPlomino.PlominoHidewhen import PlominoHidewhen
from Products.CMFPlomino.PlominoCache import PlominoCache

class PlominoForm(object):
    """
    """
    action_template = ViewPageTemplateFile('action.pt')
    
    def __init__(self, context, request):
        """Initialize adapter."""
        self.context = context
        self.request = request
        
    def __call__(self):
        """
        """
        return self
    
    def getFieldTypes(self):
        """Return a list of possible types for a field.
        """
        fieldtypes = [(pair[0], pair[1][0]) for pair in get_field_types().items()]
        fieldtypes.sort(key=lambda f:f[1])
        return fieldtypes
        
    
    def getFieldModes(self):
        """Return a list of possible modes for a field.
        """
        return FIELD_MODES

    def getField(self):
        """Return a field from the request, or the first field if empty, or None if the specified field doesn't exist. 
        """
        fieldid = self.request.get("fieldid", None)
        if fieldid:
            field = getattr(self.context, fieldid, None)
            if isinstance(field, PlominoField):
                return field
            else:
                return None
        
        fieldsList = self.context.getFormFields()
        if len(fieldsList) > 0:
            return fieldsList[0]
        else:
            return None

    def getFieldProperties(self):
        """Return properties of an action, or , if no action is given, properties filled with default values.
        """
        field = self.getField()
        if field:
            return {'fieldType': field.FieldType,
                    'fieldMode': field.FieldMode,
                    'formula': field.Formula
                    }
        else:
             return {'fieldType': 'TEXT',
                    'fieldMode': 'EDITABLE',
                    'formula': ''
                    }
        
    def addField(self):
        """Add a field to the form. 
        """
        fieldid = self.request.get("fieldid", None)
        fieldtype = self.request.get("fieldtype", "TEXT")
        fieldmode = self.request.get("fieldmode", "EDITABLE")
        fieldformula = self.request.get("fieldformula", "")
        
        # self.context is the current form
        if fieldid:
            if not hasattr(self.context, fieldid):
                self.context.invokeFactory('PlominoField', Title=fieldid, id=fieldid, FieldType=fieldtype, FieldMode=fieldmode)
                field = self.context.aq_inner.getFormField(fieldid)
                field.setFormula(fieldformula)
                field.setTitle(fieldid)
                field.at_post_create_script()
                
                self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/valid_page?type=field&value=" + fieldid + "&fieldurl=" + "/".join(field.getPhysicalPath()))
            
            else:
                self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/error_page?error=object_exists")
            
        else:
            self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/error_page?error=no_field")
            
    
    def getActions(self):
        """Returns a sorted list of actions
        """
        actions = self.context.aq_inner.getActions(None, False)
        actions = [a[0] for a in actions]
        actions.sort(key=lambda elt: elt.id.lower())
        return actions
            

    def getActionTypes(self):
        """Return a list of possible types for an action.
        """
        return ACTION_TYPES
        
    def getActionDisplay(self):
        """Return a list of possible displays for an action.
        """
        return ACTION_DISPLAY

    def getAction(self):
        """Return an action from the request, or the first action if empty, or None if the specified action doesn't exist. 
        """
        actionid = self.request.get("actionid", None)
        if actionid:
            action = getattr(self.context, actionid, None)
            if isinstance(action, PlominoAction):
                return action
            else:
                return None
        
        actionsList = self.context.getActions(None, False)
        if len(actionsList) > 0:
            return actionsList[0][0]
        else:
            return None

    def getActionProperties(self):
        """Return properties of an action, or , if no action is given, properties filled with default values.
        """
        action = self.getAction()
        if action:
            return {'title': action.Title(),
                    'actionType': action.ActionType,
                    'actionDisplay': action.ActionDisplay,
                    'content': action.Content,
                    'hideWhen': action.Hidewhen,
                    'inActionBar': action.InActionBar
                    }
        else:
             return {'title': '',
                     'actionType': 'OPENFORM',
                     'actionDisplay': 'LINK',
                     'content': '',
                     'hideWhen': '',
                     'inActionBar': False
                     }
     
    def addAction(self):
        """ Add an action to the form. 
        """
        actionid = self.request.get("actionid", None)
        title = self.request.get("actiontitle", actionid)
        actionType = self.request.get("actiontype", 'OPENFORM')
        actionDisplay = self.request.get("actiondisplay", 'LINK')
        content = self.request.get("actioncontent", '')
        hideWhen = self.request.get("actionhidewhen", '')
        inActionBar = self.request.get("actioninactionbar", None) == 'on'
        
        # self.context is the current form
        if actionid:
            if not hasattr(self.context, actionid):
                self.context.invokeFactory('PlominoAction', Title=title, id=actionid, ActionType=actionType, ActionDisplay=actionDisplay, Content=content, Hidewhen=hideWhen, InActionBar=inActionBar)
                action = getattr(self.context.aq_inner, actionid)
                action.setTitle(title)
                #action = getattr(self.context.aq_parent.aq_parent, actionid)
                #action.at_post_edit_script()
        
                self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/valid_page?type=action&value=" + actionid)
            
            else:
                self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/error_page?error=object_exists")
            
        else:
            self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/error_page?error=no_action")
        
    def getSubForms(self):
        """Returns a list of forms from the parent database, without the current form
        """
        form = self.context.aq_inner
        subforms = form.getParentDatabase().getForms()
        subforms.remove(form)
        subforms.sort(key=lambda elt: elt.id.lower())
        return subforms
    
    def getHidewhenFormulas(self):
        """Returns a sorted list of Hide-when
        """
        hw = self.context.aq_inner.getHidewhenFormulas()
        hw.sort(key=lambda elt: elt.id.lower())
        return hw
    
    def getHidewhen(self):
        """Returns a hide-when formula from the request, or the first hide-when formula if empty, or None if the specified hide-when doesn't exist. 
        """
        hidewhenid = self.request.get("hidewhenid", None)
        
        if self.request.get("create", False):
            return None
        
        if hidewhenid:
            hidewhen = getattr(self.context, hidewhenid, None)
            if isinstance(hidewhen, PlominoHidewhen):
                return hidewhen
            else:
                return None
        
        hidewhenList = self.context.getHidewhenFormulas()
        if len(hidewhenList) > 0:
            return hidewhenList[0]
        else:
            return None

    def getHidewhenProperties(self):
        """Returns properties of a hide-when formula, or, if no hide-when formula is given, properties filled with default values.
        """
        hidewhen = self.getHidewhen()
        if hidewhen:
            return { 'formula': hidewhen.Formula, 'isdynamichidewhen': hidewhen.isDynamicHidewhen }
        else:
             return { 'formula': '', 'isdynamichidewhen': False }
         
    def addHidewhen(self):
        """ Add a hide-when to the form. 
        """
        hidewhenid = self.request.get("hidewhenid", None)
        hidewhenformula = self.request.get("hidewhenformula", '')
        hidewhentype = self.request.get("hidewhentype", 'static')
        
        # self.context is the current form
        if hidewhenid:
            if not hasattr(self.context, hidewhenid):
                self.context.invokeFactory('PlominoHidewhen', Title=hidewhenid, id=hidewhenid, Formula=hidewhenformula, isDynamicHidewhen=hidewhentype=='dynamic')
                hidewhen = getattr(self.context.aq_inner, hidewhenid)
                hidewhen.setTitle(hidewhenid)
#                hidewhen.at_post_edit_script()

                self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/valid_page?type=hidewhen&value=" + hidewhenid)
            
            else:
                self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/error_page?error=object_exists")
            
        else:
            self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/error_page?error=no_hidewhen")


    #PLOMINO CACHE
    def getCacheFormulas(self):
        """Returns a sorted list of caches
        """
        hw = self.context.aq_inner.getCacheFormulas()
        hw.sort(key=lambda elt: elt.id.lower())
        return hw
    
    def getCacheFormula(self):
        """Returns a cache formula from the request, or the first cache formula if empty, or None if the specified cache doesn't exist. 
        """
        cacheid = self.request.get("cacheid", None)
        
        if self.request.get("create", False):
            return None
        
        if cacheid:
            cache = getattr(self.context, cacheid, None)
            if isinstance(cache, PlominoCache):
                return cache
            else:
                return None
        
        cacheList = self.context.getCacheFormulas()
        if len(cacheList) > 0:
            return cacheList[0]
        else:
            return None

    def getCacheProperties(self):
        """Returns properties of a cache formula, or, if no cache formula is given, properties filled with default values.
        """
        cache = self.getCacheFormula()
        if cache:
            return { 'formula': cache.Formula}
        else:
             return { 'formula': ''}
         
    def addCache(self):
        """ Add a cache to the form. 
        """
        cacheid = self.request.get("cacheid", None)
        cacheformula = self.request.get("cacheformula", '')
        
        # self.context is the current form
        if cacheid:
            if not hasattr(self.context, cacheid):
                self.context.invokeFactory('PlominoCache', Title=cacheid, id=cacheid, Formula=cacheformula)
                cache = getattr(self.context.aq_inner, cacheid)
                cache.setTitle(cacheid)

                self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/valid_page?type=cache&value=" + cacheid)
            
            else:
                self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/error_page?error=object_exists")
            
        else:
            self.request.RESPONSE.redirect(self.context.absolute_url() + "/@@tinymceplominoform/error_page?error=no_cache")



from zope import component, interface

from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class ILayer(interface.Interface):
    """Marker interface that defines a Zope 3 browser layer.
    """ 

class ITinyMCEPlominoFormView(interface.Interface):
    """Marker interface"""


class TinyMCEPlominoFormView(BrowserView):
    """
    """
    interface.implements(ITinyMCEPlominoFormView)
    
    action_template = ViewPageTemplateFile('action.pt')
    def action_form(self, **params):
        """."""
        params = {}
        return self.action_template(**params)
    
    cache_template = ViewPageTemplateFile('cache.pt')
    def cache_form(self, **params):
        """."""
        params = {}
        return self.cache_template(**params)
    
    field_template = ViewPageTemplateFile('field.pt')
    def field_form(self, **params):
        """."""
        params = {}
        return self.field_template(**params)
    
    hidewhen_template = ViewPageTemplateFile('hidewhen.pt')
    def hidewhen_form(self, **params):
        """."""
        params = {}
        return self.hidewhen_template(**params)
    
    subform_template = ViewPageTemplateFile('subform.pt')
    def subform_form(self, **params):
        """."""
        params = {}
        return self.subform_template(**params)
    
    error_template = ViewPageTemplateFile('error.pt')
    def error_page(self, **params):
        """."""
        params = {}
        return self.error_template(**params)
    
    valid_template = ViewPageTemplateFile('valid.pt')
    def valid_page(self, **params):
        """."""
        params = {}
        return self.valid_template(**params)